﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Windows;

using Zeta.Bot;
using Zeta.Common;
using Zeta.Common.Plugins;
using Zeta.Game;
using Zeta.Game.Internals;
using Zeta.Game.Internals.Actors;

namespace TaegukSpammer
{						
	public static class Logger
    {
        private static readonly log4net.ILog Logging = Zeta.Common.Logger.GetLoggerInstanceForType();

        public static void Log(string message, params object[] args)
        {
            StackFrame frame = new StackFrame(1);
            var method = frame.GetMethod();
            var type = method.DeclaringType;

            Logging.WarnFormat("[TaegukSpammer] " + string.Format(message, args), type.Name);
        }

        public static void Log(string message)
        {
            Log(message, string.Empty);			
        }
    }

    public class TaegukSpammer : IPlugin
    {
        #region Plugin Info

        public static readonly string NAME = "TaegukSpammer";
        private static readonly string AUTHOR = "piotr";
        private static readonly Version VERSION = new Version(1, 0, 0);
        private static readonly string DESCRIPTION = "Spams primary skill every 2.5 secs to keep uptime of taeguks buff";

        public string Author
        {
            get
            {
                return AUTHOR;
            }
        }
        public string Description
        {
            get
            {
                return DESCRIPTION;
            }
        }
        public string Name
        {
            get
            {
                return NAME;
            }
        }
        public Version Version
        {
            get
            {
                return VERSION;
            }
        }
        public Window DisplayWindow
		{
			get
			{
				return Config.GetDisplayWindow();
			}
		}

        public bool Equals(IPlugin other)
        {
            return Name.Equals(other.Name) && Author.Equals(other.Author) && Version.Equals(other.Version);
        }

        #endregion
		
		#region Attributes

        private bool useTaegukSpammer = false;
		private DateTime lastSpam = DateTime.MinValue;
		private DateTime combatDelayEnd = DateTime.MaxValue;

        #endregion

        #region Demonbuddy Events

        public void OnDisabled()
        {
            Logger.Log("Disabled");
            useTaegukSpammer = false;
        }

        public void OnEnabled()
        {
            Logger.Log("Enabled " + Version + " now in action!");
            useTaegukSpammer = true;
            lastSpam = DateTime.Now;
        }

        public void OnInitialize() { }

        public void OnPulse()
		{
            if (ZetaDia.Me == null)
                return;
				
			if (!ZetaDia.IsInGame || !ZetaDia.Me.IsValid || ZetaDia.IsLoadingWorld || ZetaDia.IsPlayingCutscene || ZetaDia.IsInTown || ZetaDia.Me.IsDead)
                return;
				
			if (ZetaDia.Me.IsInCombat)
				combatDelayEnd = DateTime.Now.AddMilliseconds(TaegukSpammerSettings.Instance.OOCDelay);
				
			if (combatDelayEnd == DateTime.MaxValue)
				return;
			
			if (DateTime.Now > combatDelayEnd)
			{
				Logger.Log("OutOfCombat Delay elapsed (" + TaegukSpammerSettings.Instance.OOCDelay.ToString() + "ms)");
				combatDelayEnd = DateTime.MaxValue;
				return;
			}

			if (ZetaDia.Me.HasBuff(SNOPower.ItemPassive_Unique_Gem_015_x1))
        	{
				if (DateTime.Now >= lastSpam.AddMilliseconds(TaegukSpammerSettings.Instance.Interval))
				{
					lastSpam = DateTime.Now;
					
					Buff taeguk = ZetaDia.Me.GetBuff(SNOPower.ItemPassive_Unique_Gem_015_x1);
					Logger.Log("Taeguk active (Stacks = " + taeguk.StackCount + ")");
					
					SNOPower skill = getSNOPowerBySkillID(TaegukSpammerSettings.Instance.Skill);
					if (skill != 0)
						TryCast(skill);
				}
        	}
		}

        public void OnShutdown()
        {
            useTaegukSpammer = false;
        }

        #endregion

        
        private void TryCast(SNOPower skill)
        {
        	List<SNOPower> skills = GetPlayerSkills();
        	
        	if (skills != null && skills.Contains(skill) && PowerManager.CanCast(skill))
        	{
				ZetaDia.Me.UsePower(skill, ZetaDia.Me.Position, ZetaDia.WorldId);
				Logger.Log("Casting skill: " + skill.ToString());
        	}
        	else
        		Logger.Log("Skill not ready: " + skill.ToString());
        }
        
        private SNOPower getSNOPowerBySkillID(int skillID)
        {
        	List<SNOPower> skills = GetPlayerSkills();
        	
        	if (skills != null && skills.Count > skillID)
        		return skills[skillID];
        	else
        		return 0;
        }
        
		private List<SNOPower> GetPlayerSkills()
		{
			List<SNOPower> skills = new List<SNOPower>();
	        for (int i = 0; i <= 5; i++)
	        {
				DiaActiveSkill activeSkillByIndex = ZetaDia.PlayerData.GetActiveSkillByIndex(i, ZetaDia.Me.SkillOverrideActive);
	            if (activeSkillByIndex != null)
	            {
	            	skills.Add(activeSkillByIndex.Power);
	            	Logger.Log("Skill " + i + ": " + activeSkillByIndex.Power.ToString());
	            }
	        }
	        
	        return skills;
		}
    }
    
	public static class Config
    {
        private static Window configWindow;

        public static void CloseWindow()
        {
            configWindow.Close();
        }

        public static Window GetDisplayWindow()
        {
            try
            {
                if (configWindow == null)
                {
                    configWindow = new Window();
                }

                string assemblyPath = Path.GetDirectoryName(Assembly.GetEntryAssembly().Location);
                string xamlPath = Path.Combine(assemblyPath, "Plugins", TaegukSpammer.NAME, "TaegukSpammer.Settings.xaml");
                string xamlContent = File.ReadAllText(xamlPath);
                configWindow.DataContext = TaegukSpammerSettings.Instance;
                System.Windows.Controls.UserControl mainControl =
					(System.Windows.Controls.UserControl)System.Windows.Markup.XamlReader.Load(new MemoryStream(Encoding.UTF8.GetBytes(xamlContent)));
                configWindow.Content = mainControl;
                configWindow.Width = mainControl.Width+4;
                configWindow.Height = mainControl.Height+28;
                configWindow.ResizeMode = ResizeMode.NoResize;
                configWindow.Background = System.Windows.Media.Brushes.DarkGray;
                configWindow.Title = TaegukSpammer.NAME;
                configWindow.Closed += ConfigWindow_Closed;
                #region overloads
                Demonbuddy.App.Current.Exit += ConfigWindow_Closed;
                #endregion //overloads
                return configWindow;
            }
            catch (Exception e)
            {
                Logger.Log("Unable to load Config Window!");
                return null;
            }
        }

        private static void ConfigWindow_Closed(object sender, System.EventArgs e)
        {
            TaegukSpammerSettings.Instance.Save();
            if (configWindow != null)
            {
                configWindow.Closed -= ConfigWindow_Closed;
                configWindow = null;
            }
        }
    }
}
